"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const steps_1 = require("./steps");
const utils_1 = require("./utils");
function create(projectNameArg, options) {
    return __awaiter(this, void 0, void 0, function* () {
        // 1. Get all the required project information
        const { projectDisplayName, projectPath } = yield steps_1.getProjectInfo(projectNameArg);
        const relProjectPath = utils_1.relative(projectPath);
        const template = steps_1.getTemplate(options.template);
        const installCommand = yield steps_1.getInstallCommand(options.installer);
        utils_1.log.info(`Scaffolding \`${relProjectPath}\` using  \`${template}\` template`);
        // 2. Scaffold Project
        yield steps_1.scaffoldTemplate({
            projectPath,
            template
        });
        utils_1.log.info(`Running \`${installCommand}\``);
        // 3. Install Dependencies
        try {
            yield utils_1.run(installCommand, {
                cwd: projectPath
            });
        }
        catch (err) {
            utils_1.log.failure(`Could not install dependencies. Skipping ${installCommand}`);
        }
        // 4. Set name in project's package.json
        steps_1.setNameInPackageJSON(`${projectPath}/package.json`, projectDisplayName);
        // 5. Delete `.git` (For projects scaffolded from github)
        utils_1.deleteDir(`${projectPath}/.git`);
        // 6. Log Success @todo
        utils_1.log.success(`${projectDisplayName} scaffolded successfully`);
        const runCommand = installCommand === 'yarn' ? 'yarn dev' : 'npm run dev';
        utils_1.log.info(`cd ${relProjectPath} and run \`${runCommand}\` to run the dev-server`);
    });
}
exports.default = create;
