"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setNameInPackageJSON = exports.scaffoldTemplate = exports.getTemplate = exports.getInstallCommand = exports.getProjectInfo = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const prompts_1 = __importDefault(require("prompts"));
const utils_1 = require("./utils");
/**
 * Prompts user for projectName if not defined, returns the information required related to project
 */
// eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
const getProjectInfo = (projectNameArg) => __awaiter(void 0, void 0, void 0, function* () {
    let projectName = '';
    if (!projectNameArg) {
        projectName = (yield prompts_1.default({
            type: 'text',
            message: 'Enter Name of your project',
            name: 'projectName',
            initial: 'hello-abell'
        })).projectName;
    }
    else {
        projectName = projectNameArg;
    }
    if (!projectName) {
        throw new Error(utils_1.log.failure('Project name is required', false));
    }
    const projectSlugName = projectName.toLowerCase().replace(/ |_/g, '-');
    const projectPath = path_1.default.join(process.cwd(), projectSlugName);
    const projectDisplayName = path_1.default.basename(projectPath);
    if (fs_1.default.existsSync(projectPath)) {
        // oops. Can be an issue
        if (fs_1.default.readdirSync(projectPath).length !== 0) {
            // Not an empty directory so break!
            console.error(`${utils_1.colors.red('>> ')} The directory already exists and is not an empty directory`);
            process.exit(0);
        }
    }
    return { projectDisplayName, projectPath };
});
exports.getProjectInfo = getProjectInfo;
/**
 * Prompts user to choose package installer if not defined
 */
const getInstallCommand = (installerVal) => __awaiter(void 0, void 0, void 0, function* () {
    if (!installerVal) {
        // if installer flag is undefined, ask user.
        const answers = yield prompts_1.default({
            type: 'select',
            message: 'Select Installer',
            name: 'installer',
            choices: [
                {
                    title: 'npm',
                    value: 'npm install'
                },
                {
                    title: 'yarn',
                    value: 'yarn'
                }
            ]
        });
        installerVal = answers.installer;
    }
    if (installerVal === 'yarn') {
        return 'yarn';
    }
    else {
        return 'npm install';
    }
});
exports.getInstallCommand = getInstallCommand;
/**
 * Some validations on top of template names
 */
const getTemplate = (templateVal) => {
    // return default when value is not defined
    if (!templateVal)
        return 'default';
    if (templateVal === 'default' || templateVal === 'minimal') {
        // 'default' and 'minimal' are valid templates. Return them as it is
        return templateVal;
    }
    // when `--template abelljs/abell-starter-portfolio`
    if (!templateVal.startsWith('https://github.com/')) {
        // If template value is `abelljs/abell-starter-portfolio`, add https://github.com before it.
        return 'https://github.com/' + templateVal;
    }
    // when `--template https://github.com/abelljs/abell-starter-portfolio`
    return templateVal;
};
exports.getTemplate = getTemplate;
const scaffoldTemplate = ({ projectPath, template }) => __awaiter(void 0, void 0, void 0, function* () {
    if (template === 'default' || template === 'minimal') {
        // copy default template from templates directory
        const templatesDir = path_1.default.join(__dirname, '..', 'templates');
        const templatePath = path_1.default.join(templatesDir, template);
        utils_1.copyFolderSync(templatePath, projectPath, [
            path_1.default.join(templatePath, 'node_modules'),
            path_1.default.join(templatePath, 'yarn.lock'),
            path_1.default.join(templatePath, 'dist')
        ]);
    }
    else {
        // Execute git clone
        try {
            const errorCode = yield utils_1.run(`git clone ${template} ${projectPath}`);
            if (errorCode === 1) {
                throw new Error(utils_1.log.failure('Git clone failed', false));
            }
        }
        catch (err) {
            throw err;
        }
    }
});
exports.scaffoldTemplate = scaffoldTemplate;
const setNameInPackageJSON = (packagePath, appName) => {
    try {
        // eslint-disable-next-line @typescript-eslint/no-var-requires
        const packageJSON = require(utils_1.normalizePath(packagePath));
        packageJSON.name = appName;
        fs_1.default.writeFileSync(packagePath, JSON.stringify(packageJSON, null, 2));
    }
    catch (err) {
        // Do nothing. Skip the step if error.
    }
};
exports.setNameInPackageJSON = setNameInPackageJSON;
