"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteDir = exports.copyFolderSync = exports.run = exports.normalizePath = exports.log = exports.colors = exports.relative = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const isWindows = /^win/.test(process.platform);
const relative = (pathString) => path_1.default.relative(process.cwd(), pathString);
exports.relative = relative;
exports.colors = {
    red: (message) => `\u001b[1m\u001b[31m${message}\u001b[39m\u001b[22m`,
    cyan: (message) => `\u001b[36m${message}\u001b[39m`,
    green: (message) => `\u001b[1m\u001b[32m${message}\u001b[39m\u001b[22m`
};
exports.log = {
    success: (message) => console.log(`${exports.colors.green('✓')} ${message}`),
    failure: (message, shouldLog = true) => {
        const coloredMessage = `${exports.colors.red('✗ [create-abell]:')} ${message}`;
        if (shouldLog) {
            console.log(coloredMessage);
        }
        return coloredMessage;
    },
    info: (message) => console.log(`${exports.colors.cyan('>>')} ${message}`)
};
const normalizePath = (pathString) => pathString.split('/').join(path_1.default.sep);
exports.normalizePath = normalizePath;
const windowsifyCommand = (command) => {
    if (!isWindows) {
        return command;
    }
    return command.replace('npm', 'npm.cmd').replace('yarn', 'yarn.cmd');
};
function run(command, { cwd } = { cwd: process.cwd() }) {
    return __awaiter(this, void 0, void 0, function* () {
        const commandArgs = windowsifyCommand(command).split(' ');
        return new Promise((resolve, reject) => {
            try {
                const child = child_process_1.spawn(commandArgs[0], [...commandArgs.slice(1)], {
                    cwd,
                    stdio: 'inherit'
                });
                child.on('close', (code) => {
                    if (code === 0) {
                        resolve(0);
                    }
                    else {
                        console.log(`oops ${command} failed`);
                        // eslint-disable-next-line prefer-promise-reject-errors
                        reject(1);
                    }
                });
            }
            catch (err) {
                // eslint-disable-next-line prefer-promise-reject-errors
                reject(1);
            }
        });
    });
}
exports.run = run;
// fs methods
/**
 * Recursively creates the path
 * @param {String} pathToCreate path that you want to create
 */
function createPathIfAbsent(pathToCreate) {
    // prettier-ignore
    pathToCreate
        .split(path_1.default.sep)
        .reduce((prevPath, folder) => {
        const currentPath = path_1.default.join(prevPath, folder, path_1.default.sep);
        if (!fs_1.default.existsSync(currentPath)) {
            fs_1.default.mkdirSync(currentPath);
        }
        return currentPath;
    }, '');
}
/**
 *
 * @param {string} from - Path to copy from
 * @param {string} to - Path to copy to
 * @param {string[]} ignore - files/directories to ignore
 * @param {boolean} ignoreEmptyDirs - Ignore empty directories while copying
 * @return {void}
 */
function copyFolderSync(from, to, ignore = [], ignoreEmptyDirs = true) {
    if (ignore.includes(from)) {
        return;
    }
    const fromDirectories = fs_1.default.readdirSync(from);
    createPathIfAbsent(to);
    fromDirectories.forEach((element) => {
        const fromElement = path_1.default.join(from, element);
        const toElement = path_1.default.join(to, element);
        if (fs_1.default.lstatSync(fromElement).isFile()) {
            if (!ignore.includes(fromElement)) {
                fs_1.default.copyFileSync(fromElement, toElement.replace(/gitignore/g, '.gitignore'));
            }
        }
        else {
            copyFolderSync(fromElement, toElement, ignore);
            if (fs_1.default.existsSync(toElement) && ignoreEmptyDirs) {
                try {
                    fs_1.default.rmdirSync(toElement);
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any
                }
                catch (err) {
                    if (err.code !== 'ENOTEMPTY')
                        throw err;
                }
            }
        }
    });
}
exports.copyFolderSync = copyFolderSync;
function deleteDir(unNormalizedPathToRemove) {
    const pathToRemove = exports.normalizePath(unNormalizedPathToRemove);
    if (fs_1.default.existsSync(pathToRemove)) {
        fs_1.default.readdirSync(pathToRemove).forEach((file) => {
            const curPath = path_1.default.join(pathToRemove, file);
            if (fs_1.default.lstatSync(curPath).isDirectory()) {
                // recurse
                deleteDir(curPath);
            }
            else {
                // delete file
                fs_1.default.unlinkSync(curPath);
            }
        });
        fs_1.default.rmdirSync(pathToRemove);
    }
}
exports.deleteDir = deleteDir;
